import urllib
from enum import Enum
from logging import Logger, getLogger
from urllib.error import HTTPError, URLError
from http.client import IncompleteRead


class SkipReason(Enum):
    UNICODE = 1  #: Unicode in response
    HTTP = 2  #: HTTP error
    SSL = 3  #: Invalid SSL cert
    PACKET = 4  #: Failed to finish reading
    URL = 5  #: Most likely a dead URL


def patch(
    url: str,
    logger: Logger = getLogger(),
    headers: dict = {}
):
    """
    Make a PATCH request.

    :param url: The URL
    :type url: str
    :param logger: The logger to use
    :type logger: logging.Logger
    :param headers: Custom headers to use
    :type headers: dict
    :return: The page's content, or a int (a :meth:`SkipReason`)
    """
    return _common(url, "PATCH", logger, headers)


def get(
    url: str,
    logger: Logger = getLogger(),
    headers: dict = {}
):
    """
    Make a GET request.

    :param url: The URL
    :type url: str
    :param logger: The logger to use
    :type logger: logging.Logger
    :param headers: Custom headers to use
    :type headers: dict
    :return: The page's content, or a int (one of SkipReason's members)
    """
    return _common(url, "GET", logger, headers)


def _common(url, method: str, logger: Logger, headers: dict):
    logger.info(f"Making request to {url}.")
    urllib.request.urlcleanup()
    try:
        return urllib.request.urlopen(
            urllib.request.Request(
                url,
                method,
                headers=headers
            )
        ).read()
    except HTTPError:
        return SkipReason.HTTP
    except URLError:
        return SkipReason.SSL
    except UnicodeError:
        return SkipReason.UNICODE
    except IncompleteRead:
        return SkipReason.PACKET
    except ConnectionResetError:
        return SkipReason.HTTP
    except ConnectionAbortedError:
        return SkipReason.PACKET
    except ValueError:
        return SkipReason.URL
